import unittest

from .cn2an import Cn2An


class Cn2anTest(unittest.TestCase):
    def setUp(self):
        self.strict_data_dict = {
            "一": 1,
            "两": 2,
            "十": 10,
            "十一": 11,
            "一十一": 11,
            "一百一十一": 111,
            "一千一百一十一": 1111,
            "一万一千一百一十一": 11111,
            "一十一万一千一百一十一": 111111,
            "一百一十一万一千一百一十一": 1111111,
            "一千一百一十一万一千一百一十一": 11111111,
            "一亿一千一百一十一万一千一百一十一": 111111111,
            "一十一亿一千一百一十一万一千一百一十一": 1111111111,
            "一百一十一亿一千一百一十一万一千一百一十一": 11111111111,
            "一千一百一十一亿一千一百一十一万一千一百一十一": 111111111111,
            "壹": 1,
            "拾": 10,
            "拾壹": 11,
            "壹拾壹": 11,
            "壹佰壹拾壹": 111,
            "壹仟壹佰壹拾壹": 1111,
            "壹万壹仟壹佰壹拾壹": 11111,
            "壹拾壹万壹仟壹佰壹拾壹": 111111,
            "壹佰壹拾壹万壹仟壹佰壹拾壹": 1111111,
            "壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 11111111,
            "壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 111111111,
            "壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 1111111111,
            "壹佰壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 11111111111,
            "壹仟壹佰壹拾壹亿壹仟壹佰壹拾壹万壹仟壹佰壹拾壹": 111111111111,
            "一百零一": 101,
            "一千零一": 1001,
            "一千零一十一": 1011,
            "一千一百零一": 1101,
            "一万零一": 10001,
            "一万零一十一": 10011,
            "一万零一百一十一": 10111,
            "一十万零一": 100001,
            "一百万零一": 1000001,
            "一千万零一": 10000001,
            "一千零一万一千零一": 10011001,
            "一千零一万零一": 10010001,
            "一亿零一": 100000001,
            "一十亿零一": 1000000001,
            "一百亿零一": 10000000001,
            "一千零一亿一千零一万一千零一": 100110011001,
            "一千亿一千万一千零一": 100010001001,
            "一千亿零一": 100000000001,
            "零点零零零零零零零零零零零零零零一": 0.000000000000001,
            "零点零零零零零零零零零零零零零一": 0.00000000000001,
            "零点零零零零零零零零零零零零一": 0.0000000000001,
            "零点零零零零零零零零零零零一": 0.000000000001,
            "零点零零零零零零零零零零一": 0.00000000001,
            "零点零零零零零零零零零一": 0.0000000001,
            "零点零零零零零零零零一": 0.000000001,
            "零点零零零零零零零一": 0.00000001,
            "零点零零零零零零一": 0.0000001,
            "零点零零零零零一": 0.000001,
            "零点零零零零一": 0.00001,
            "零点零零零一": 0.0001,
            "零点零零一": 0.001,
            "零点零一": 0.01,
            "零点一": 0.1
        }

        self.normal_data_dict = {
            "十万": 100000,
            "十万零一": 100001,
            "十亿零一万零一": 1000010001,
            "一一": 11,
            "一一一": 111,
            "一一一一": 1111,
            "拾万零壹": 100001,
            "拾亿零壹万零壹": 1000010001,
            "壹壹": 11,
            "壹壹壹": 111,
            "壹壹壹壹": 1111,
            "零点": 0,
            "零点零": 0,
            "零点零零": 0,
            "点零": 0,
            "点一": 0.1,
            "一七二零": 1720,
            "一七二零点一": 1720.1,
            "一七二零点一三四": 1720.134,
            "三万五": 35000,
            "两千六": 2600,
            "一百二": 120
        }
        self.normal_data_dict.update(self.strict_data_dict)

        self.error_normal_datas = [
            "零点点",
            "零点零大"
        ]

        self.error_strict_datas = [
            "一一",
            "壹壹",
            "零点",
            "点零",
            "点一",
        ]
        self.error_strict_datas.extend(self.error_normal_datas)

        self.ca = Cn2An()

    def test_cn2an(self):
        for strict_item in self.strict_data_dict.keys():
            self.assertEqual(self.ca.cn2an(strict_item, "strict"),
                             self.strict_data_dict[strict_item])

        for normal_item in self.normal_data_dict.keys():
            self.assertEqual(self.ca.cn2an(normal_item, "normal"),
                             self.normal_data_dict[normal_item])

        for error_strict_item in self.error_strict_datas:
            try:
                self.ca.cn2an(error_strict_item)
            except ValueError as e:
                self.assertEqual(type(e), ValueError)
            else:
                raise Exception(f'ValueError not raised: {error_strict_item}')

        for error_normal_item in self.error_normal_datas:
            try:
                self.ca.cn2an(error_normal_item)
            except ValueError as e:
                self.assertEqual(type(e), ValueError)
            else:
                raise Exception(f'ValueError not raised: {error_normal_item}')


if __name__ == '__main__':
    unittest.main()



from . import utils


class An2Cn(object):
    def __init__(self):
        self.conf = utils.get_default_conf()

    def an2cn(self, inputs=None, mode="low"):
        if inputs is not None:
            if mode not in ["low", "up", "rmb"]:
                raise ValueError("mode 仅支持 low up rmb smart 四种！")

            # 将数字转化为字符串
            if not isinstance(inputs, str):
                inputs = self.convert_number_to_string(inputs)

            # 检查数据是否有效
            self.check_inputs_is_valid(inputs)

            # 切割整数部分和小数部分
            split_result = inputs.split(".")
            len_split_result = len(split_result)
            if len_split_result == 1:
                # 不包含小数的输入
                integer_data = split_result[0]
                if mode == "rmb":
                    output = self.integer_convert(integer_data, "up") + "元整"
                else:
                    output = self.integer_convert(integer_data, mode)
            elif len_split_result == 2:
                # 包含小数的输入
                integer_data = split_result[0]
                decimal_data = split_result[1]
                if mode == "rmb":
                    int_data = self.integer_convert(integer_data, "up")
                    dec_data = self.decimal_convert(decimal_data, "up")
                    len_dec_data = len(dec_data)

                    if len_dec_data == 0:
                        output = int_data + "元整"
                    elif len_dec_data == 1:
                        raise ValueError(f"异常输出：{dec_data}")
                    elif len_dec_data == 2:
                        if dec_data[1] != "零":
                            if int_data == "零":
                                output = dec_data[1] + "角"
                            else:
                                output = int_data + "元" + dec_data[1] + "角"
                        else:
                            output = int_data + "元整"
                    else:
                        if dec_data[1] != "零":
                            if dec_data[2] != "零":
                                if int_data == "零":
                                    output = dec_data[1] + "角" + dec_data[2] + "分"
                                else:
                                    output = int_data + "元" + dec_data[1] + "角" + dec_data[2] + "分"
                            else:
                                if int_data == "零":
                                    output = dec_data[1] + "角"
                                else:
                                    output = int_data + "元" + dec_data[1] + "角"
                        else:
                            if dec_data[2] != "零":
                                if int_data == "零":
                                    output = dec_data[2] + "分"
                                else:
                                    output = int_data + "元" + "零" + dec_data[2] + "分"
                            else:
                                output = int_data + "元整"

                else:
                    output = self.integer_convert(integer_data, mode) + self.decimal_convert(decimal_data, mode)
            else:
                raise ValueError(f"输入格式错误：{inputs}！")
        else:
            raise ValueError(f"输入数据为空！") 

        return output

    @staticmethod
    def check_inputs_is_valid(check_data):
        # 检查输入数据是否在规定的字典中
        all_check_keys = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "."]
        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(f"输入的数据不在转化范围内：{data}！")

    @staticmethod
    def convert_number_to_string(number_data):
        # python 会自动把 0.00005 转化成 5e-05，因此 str(0.00005) != "0.00005"
        string_data = str(number_data)
        if "e" in string_data:
            string_data_list = string_data.split("e")
            string_key = string_data_list[0]
            string_value = string_data_list[1]
            if string_value[0] == "-":
                string_data = "0." + "0"*(int(string_value[1:])-1) + string_key
            else:
                string_data = string_key + "0"*int(string_value)

        return string_data

    def integer_convert(self, integer_data, mode):
        numeral_list = self.conf[f"number_{mode}"]
        unit_list = self.conf[f"unit_{mode}"]

        # 去除前面的 0，比如 007 => 7
        integer_data = str(int(integer_data))

        len_integer_data = len(integer_data)
        
        if len_integer_data > len(unit_list):
            raise ValueError("超出数据范围，最长支持 16 位")

        output_an = ""
        for i, d in enumerate(integer_data):
            if int(d):
                output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
            else:
                if not (len_integer_data - i - 1) % 4:
                    output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
                
                if i > 0 and not output_an[-1] == "零":
                    output_an += numeral_list[int(d)]

        output_an = output_an.replace("零零", "零").replace("零万", "万").replace("零亿", "亿").replace("亿万", "亿").strip("零")

        # 解决「一十几」和「壹拾几」问题
        if output_an[:2] in ["一十", "壹拾"]:
            output_an = output_an[1:]
            
        # 0 - 1 之间的小数
        if not output_an:
            output_an = "零"
            
        return output_an

    def decimal_convert(self, decimal_data, mode):
        len_decimal_data = len(decimal_data) 

        if len_decimal_data > 15:
            print(f"warning: 小数部分长度为{len_decimal_data}，超过15位有效精度长度，将自动截取前15位！")
            decimal_data = decimal_data[:15]

        if len_decimal_data:
            output_an = "点"
        else:
            output_an = ""

        numeral_list = self.conf[f"number_{mode}"]

        for data in decimal_data:
            output_an += numeral_list[int(data)]

        return output_an



import re

from . import utils


class Cn2An(object):
    def __init__(self):
        self.conf = utils.get_default_conf()

    def cn2an(self, inputs=None, mode="strict"):
        if inputs is not None:
            # 检查转换模式是否有效
            if mode not in ["strict", "normal", "smart"]:
                raise ValueError("mode 仅支持 strict normal smart 三种！")

            # 检查输入数据是否有效
            data_type = self.check_input_data_is_valid(inputs, mode)

            if data_type == "integer":
                # 不包含小数的输入
                output = self.integer_convert(inputs)
            elif data_type == "decimal":
                # 包含小数的输入
                integer_data, decimal_data = inputs.split("点")
                output = self.integer_convert(integer_data) + self.decimal_convert(decimal_data)
            elif data_type == "all_num":
                output = self.direct_convert(inputs)
            else:
                raise ValueError(f"输入格式错误：{inputs}！")
        else:
            raise ValueError("输入数据为空！")

        return output

    def check_input_data_is_valid(self, check_data, mode):
        # 检查输入数据是否在规定的字典中
        all_check_keys = list(self.conf["number_unit"].keys())
        all_check_keys.append("点")

        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(f"输入的数据不在转化范围内：{data}！")

        if "点" in check_data:
            split_data = check_data.split("点")
            if len(split_data) == 2:
                integer_data, decimal_data = split_data
            else:
                raise ValueError("数据中包含不止一个 点！")
        else:
            integer_data = check_data
            decimal_data = None

        all_num = "".join(set(self.conf["number_low"] + self.conf["number_up"])) + "两"
        all_unit = "".join(set(self.conf["unit_low"] + self.conf["unit_up"]))

        # 整数部分检查
        ptn_normal = re.compile(
            f"(([{all_num}十拾]+[{all_unit}]+)+零?[{all_num}]|([{all_num}十拾]+[{all_unit}]+)+|[十拾][{all_num}]|[{all_num}]|[十拾])$")
        re_normal = ptn_normal.search(integer_data)

        if re_normal:
            if re_normal.group() != integer_data:
                if mode == "strict":
                    raise ValueError(f"不符合格式的数据：{integer_data}")
                elif mode == "normal":
                    # 纯数字情况
                    ptn_all_num = re.compile(f"[{all_num}]+")
                    re_all_num = ptn_all_num.search(integer_data)
                    if re_all_num:
                        if re_all_num.group() != integer_data:
                            raise ValueError(f"不符合格式的数据：{integer_data}")
                        else:
                            return "all_num"
                else:
                    raise ValueError(f"不符合格式的数据：{integer_data}")
            else:
                if decimal_data:
                    return "decimal"
                else:
                    if check_data[-1] == "点":
                        if mode == "strict":
                            raise ValueError(f"不符合格式的数据：{check_data}")
                        elif mode == "normal":
                            return "decimal"
                    else:
                        return "integer"
        else:
            if mode == "strict":
                raise ValueError(f"不符合格式的数据：{integer_data}")
            elif mode == "normal":
                if decimal_data:
                    return "decimal"
                else:
                    raise ValueError(f"不符合格式的数据：{integer_data}")
            else:
                raise ValueError(f"不符合格式的数据：{integer_data}")

    def integer_convert(self, integer_data):
        all_num = "".join(set(self.conf["number_low"] + self.conf["number_up"])) + "两"
        ptn_speaking_mode = re.compile(f"^[{all_num}][万千百][{all_num}]$")
        result = ptn_speaking_mode.search(integer_data)

        if result:
            high_num = self.conf["number_unit"].get(integer_data[0]) * self.conf["number_unit"].get(integer_data[1])
            low_num = self.conf["number_unit"].get(integer_data[2]) * self.conf["number_unit"].get(integer_data[1])/10
            output_integer = high_num + low_num
        else:
            output_integer = 0
            unit_value = 1
            ten_thousand_unit_key = 1

            for index in range(len(integer_data) - 1, -1, -1):
                unit_key = self.conf["number_unit"].get(integer_data[index])

                if unit_key < 10:
                    output_integer += unit_value * unit_key
                else:
                    if unit_key % 10000 == 0:
                        if unit_key > ten_thousand_unit_key:
                            ten_thousand_unit_key = unit_key
                        else:
                            ten_thousand_unit_key = ten_thousand_unit_key * unit_key

                    if unit_key > unit_value:
                        unit_value = unit_key
                    else:
                        unit_value = ten_thousand_unit_key * unit_key

                    if index == 0:
                        output_integer += unit_value

        return output_integer

    def decimal_convert(self, decimal_data):
        len_decimal_data = len(decimal_data)

        if len_decimal_data > 15:
            print("warning: 小数部分长度为{}，超过15位有效精度长度，将自动截取前15位！".format(
                len_decimal_data))
            decimal_data = decimal_data[:15]
            len_decimal_data = 15

        output_decimal = 0
        for index in range(len(decimal_data)-1, -1, -1):
            unit_key = self.conf["number_unit"].get(decimal_data[index])
            output_decimal += unit_key * 10 ** -(index + 1)

        # 处理精度溢出问题
        output_decimal = round(output_decimal, len_decimal_data)

        return output_decimal

    def direct_convert(self, data):
        output_data = 0
        if "点" in data:
            point_index = data.index("点")
            for index_integer in range(point_index - 1, -1, -1):
                unit_key = self.conf["number_unit"].get(data[index_integer])
                output_data += unit_key * 10 ** (point_index - index_integer - 1)

            for index_decimal in range(len(data)-1, point_index, -1):
                unit_key = self.conf["number_unit"].get(data[index_decimal])
                output_data += unit_key * 10 ** -(index_decimal - point_index)

            # 处理精度溢出问题
            output_data = round(output_data, len(data) - point_index)
        else:
            for index in range(len(data)-1, -1, -1):
                unit_key = self.conf["number_unit"].get(data[index])
                output_data += unit_key * 10 ** (len(data)-index-1)

        return output_data



import unittest

from .an2cn import An2Cn


class An2CnTest(unittest.TestCase):
    def setUp(self):
        self.error_input_data = [
            "123.1.1",
            "0.1零"
        ]
        self.integer_data = {
            0: ["零", "零", "零元整"],
            1: ["一", "壹", "壹元整"],
            11: ["十一", "拾壹", "拾壹元整"],
            1000000: ["一百万", "壹佰万", "壹佰万元整"],
            1000054: ["一百万零五十四", "壹佰万零伍拾肆", "壹佰万零伍拾肆元整"],
            31000054: ["三千一百万零五十四", "叁仟壹佰万零伍拾肆", "叁仟壹佰万零伍拾肆元整"],
            11.: ["十一", "拾壹", "拾壹元整"]
        }
        self.integer_decimal_data = {
            0.000500050005005: ["零点零零零五零零零五零零零五零零五", "零点零零零伍零零零伍零零零伍零零伍", "零元整"],
            0.00005: ["零点零零零零五", "零点零零零零伍", "零元整"],
            0.4321: ["零点四三二一", "零点肆叁贰壹", "肆角叁分"],
            1000054.4321: ["一百万零五十四点四三二一", "壹佰万零伍拾肆点肆叁贰壹", "壹佰万零伍拾肆元肆角叁分"],
            1.01: ["一点零一", "壹点零壹", "壹元零壹分"],
            1.10: ["一点一零", "壹点壹零", "壹元壹角"],
            1.00: ["一点零零", "壹点零零", "壹元整"],
            1.1: ["一点一", "壹点壹", "壹元壹角"],
            1.0: ["一点零", "壹点零", "壹元整"],
            0.01: ["零点零一", "零点零壹", "壹分"],
            0.10: ["零点一零", "零点壹零", "壹角"],
            0.00: ["零点零零", "零点零零", "零元整"],
            0.1: ["零点一", "零点壹", "壹角"],
            0.0: ["零点零", "零点零", "零元整"]
        }

        self.ac = An2Cn()

    def test_an2cn(self):
        with self.assertRaises(ValueError):
            for error_data in self.error_input_data:
                self.ac.an2cn(error_data)

        for integer_item in self.integer_data.keys():
            self.assertEqual(self.ac.an2cn(integer_item),
                             self.integer_data[integer_item][0])
            self.assertEqual(self.ac.an2cn(integer_item, "low"),
                             self.integer_data[integer_item][0])
            self.assertEqual(self.ac.an2cn(integer_item, "up"),
                             self.integer_data[integer_item][1])
            self.assertEqual(self.ac.an2cn(integer_item, "rmb"),
                             self.integer_data[integer_item][2])

        for integer_decimal_item in self.integer_decimal_data.keys():
            self.assertEqual(self.ac.an2cn(integer_decimal_item),
                             self.integer_decimal_data[integer_decimal_item][0])
            self.assertEqual(self.ac.an2cn(integer_decimal_item, "low"),
                             self.integer_decimal_data[integer_decimal_item][0])
            self.assertEqual(self.ac.an2cn(integer_decimal_item, "up"),
                             self.integer_decimal_data[integer_decimal_item][1])
            self.assertEqual(self.ac.an2cn(integer_decimal_item, "rmb"),
                             self.integer_decimal_data[integer_decimal_item][2])


if __name__ == '__main__':
    unittest.main()



from setuptools import setup
from setuptools import find_packages

from cn2an import version


setup(
    name="cn2an",
    version=version.VERSION,
    author="HaveTwoBrush",
    author_email="kinggreenhall@gmail.com",
    url="https://github.com/HaveTwoBrush/cn2an",
    packages=find_packages(),
    include_package_data=True,
    install_requires=open("./requirements.txt", "r", encoding="utf-8").read().splitlines(),
    description="Convert Chinese numerals and Arabic numerals.",
    long_description=open("./README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



import os

import yaml


def get_default_conf():
    with open(f"{os.path.dirname(__file__)}/config.yaml", "r", encoding="utf-8") as f_config:
        config_data = yaml.load(f_config, Loader=yaml.FullLoader)
    return config_data
